<?php declare(strict_types=1);

namespace Convo\Proto;

use Convo\Core\IAdminUserDataProvider;
use Convo\Core\DataItemNotFoundException;

class AdminUserDao implements IAdminUserDataProvider
{

	private $_basePath;

	/**
	 * Logger
	 *
	 * @var \Psr\Log\LoggerInterface
	 */
	private $_logger;

	public function __construct( \Psr\Log\LoggerInterface $logger, $basePath)
	{
		$this->_logger		=	$logger;
		$this->_basePath	=	\Convo\Core\Util\StrUtil::removeTrailingSlashes( $basePath);
	}

	/**
	 * Find and return a proto admin user
	 *
	 * @param string $username Either the email or the actual username of the user
	 * @return ProtoAdminUser
	 */
	public function findUser( $username)
	{
		$users	=	$this->_getUsersData();

		$found_data = null;

		foreach ( $users as $user_data) {
			if ( $user_data['username'] === $username) {
				$found_data = $user_data;
			}
			if ( $user_data['email'] === $username) {
				$found_data = $user_data;
			}
		}

		if (!$found_data) {
		    throw new DataItemNotFoundException( 'User ['.$username.'] not found');
		}

		return new ProtoAdminUser(
			$found_data['user_id'],
			$found_data['username'],
			$found_data['name'],
			$found_data['email'],
			$found_data['password']
		);
	}

	public function getUsers()
    {
        return $this->_getUsersData();
    }

    private function _getUsersData()
	{
		$full_path	=	$this->_basePath.'/users/_users.json';

		if ( !is_file( $full_path)) {
			throw new \Exception( 'Users file not found at ['.$full_path.']');
		}

		$data	=	json_decode( file_get_contents( $full_path), true);

		if ( $data === false) {
			throw new \Exception( 'Invalid users definition in ['.$full_path.']. Reason ['.json_last_error().']['.json_last_error_msg().']');
		}

		return $data;
	}

	/**
	 * {@inheritDoc}
	 * @see \Convo\Core\IAdminUserDataProvider::getPlatformConfig()
	 */
	public function getPlatformConfig( $userId)
	{
		$full_path = $this->_basePath.'/config/'.$userId.'_platform_config.json';

		if (!is_file($full_path)) {
		    return IAdminUserDataProvider::BASE_CONFIG;
		}

		return $this->_getJson($full_path);
	}

	/**
	 * {@inheritDoc}
	 * @see \Convo\Core\IAdminUserDataProvider::updatePlatformConfig()
	 */
	public function updatePlatformConfig( $userId, $config)
	{
		$full_path = $this->_basePath.'/config/'.$userId.'_platform_config.json';

		try {
		    $existing =   $this->_getJson($full_path);
		} catch ( DataItemNotFoundException $e) {
		    $this->_logger->notice( $e->getMessage());
		    $existing =   [];
		}
		
		$data = array_replace_recursive($existing, $config);

		file_put_contents($full_path, json_encode($data, JSON_PRETTY_PRINT));

		return $data;
	}

	// UTIL
	public function __toString()
	{
		return get_class( $this).'[]';
	}

	/**
	 * Open and return a decoded JSON file
	 *
	 * @param string $path
	 * @return array
	 * @throws DataItemNotFoundException JSON decoding failed.
	 */
	private function _getJson($path)
	{
	    if ( !is_file( $path)) {
	        throw new DataItemNotFoundException('File not exists ['.$path.']');
	    }
	    
		$data = file_get_contents($path);

		if ($data === false) {
			throw new DataItemNotFoundException('Could not open ['.$path.'] for reading.');
		}


		$data = json_decode($data, true);

		if (json_last_error() !== JSON_ERROR_NONE) {
			throw new \Exception('JSON decoding failed ['.json_last_error_msg().']');
		}

		return $data;
	}

	private function _initBaseConfig($userId)
	{
		$full_path = $this->_basePath.'/config/'.$userId.'_platform_config.json';
		file_put_contents($full_path, json_encode( IAdminUserDataProvider::BASE_CONFIG, JSON_PRETTY_PRINT));
	}
}
