<?php declare(strict_types=1);

namespace Convo\Proto;

use Convo\Core\DataItemNotFoundException;
use Convo\Core\Publish\IPlatformPublisher;
use Convo\Core\Rest\RestSystemUser;
use Psr\Http\Server\RequestHandlerInterface;

class OAuthRestHandler implements RequestHandlerInterface
{

    /**
     * @var \Convo\Proto\ServiceUserDao
     */
    private $_serviceUserDao;

    /**
     * @var \Convo\Core\Util\IHttpFactory
     */
    private $_httpFactory;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    private $_logger;

    /**
     * @var \Convo\Core\IServiceDataProvider
     */
    private $_convoServiceDataProvider;

    public function __construct( $httpFactory, $logger, $serviceUserDao, $convoServiceDataProvider)
    {
    	$this->_logger				        = $logger;
    	$this->_httpFactory			        = $httpFactory;
        $this->_serviceUserDao		        = $serviceUserDao;
        $this->_convoServiceDataProvider	= $convoServiceDataProvider;
    }

    public function handle(\Psr\Http\Message\ServerRequestInterface $request): \Psr\Http\Message\ResponseInterface
    {
    	$info			=	new \Convo\Core\Rest\RequestInfo( $request);

    	if ( $info->get() && $route = $info->route( 'oauth/{type}/{serviceId}'))
    	{
    		return $this->_handleOauthPathTypeGet( $request, $route->get('type'), $route->get('serviceId'));
    	}

    	if ( $info->post() && $route = $info->route( 'token/{type}/{serviceId}'))
    	{
    		return $this->_handleTokenPathTypePost($request, $route->get('type'), $route->get('serviceId'));
    	}

    	throw new \Convo\Core\Rest\NotFoundException( 'Could not map ['.$info.']');
    }

    private function _handleOauthPathTypeGet(\Psr\Http\Message\ServerRequestInterface $request, $type, $serviceId)
    {
        $params			=	$request->getQueryParams();

        $user_id        =   $params['user_id'] ?? null;

        if (!$user_id) {
            return $this->_httpFactory->buildResponse([
                "message" => "Missing user_id query parameter."
            ], 400);
        }

        $state 			=	$params['state'] ?? null;
        $client_id 		=	$params['client_id'] ?? null;
        $response_type	=	$params['response_type'] ?? null;
        $scope			=	$params['scope'] ?? null;
        $redirect_uri	=	$params['redirect_uri'] ?? null;

        $this->_logger->debug("Got params [$state][$client_id][$response_type][$scope][$redirect_uri]");

        try {
            $user = $this->_serviceUserDao->getUserById($user_id);

            $code = $this->_generateAuthCodeForUser($user);

            if (isset($user['authCode'][$serviceId][$type]['code']) &&
                    $user['authCode'][$serviceId][$type]['code'] === $code &&
                    $user['authCode'][$serviceId][$type]['redeemed'] === true)
            {
                return $this->_httpFactory->buildResponse(['message' => 'Code has already been redeemed.'], 400);
            }

            $this->_serviceUserDao->updateUser($user['userId'], [
                'authCode' => [
                    $serviceId => [
                        $type => [
                            'code' => $code,
                            'redeemed' => false
                        ]
                    ]
                ]
            ]);

            return $this->_httpFactory->buildResponse([], 302, [
                'Location' => $redirect_uri."?state={$state}&code={$code}"
            ]);
        } catch (DataItemNotFoundException $e) {
            return $this->_httpFactory->buildResponse([], 401);
        }
    }

    private function _handleTokenPathTypePost(\Psr\Http\Message\ServerRequestInterface $request, $type, $serviceId)
    {
        $json = $request->getParsedBody();

        $this->_logger->debug('Got JSON ['.print_r($json, true).']');

        $grant_type = $json['grant_type'] ?? null;

        if (!$grant_type) {
            throw new \Exception('Missing grant type.');
        }

        $authorizationHeaderValue = $request->getHeaderLine('authorization');
        $credentialsValid = $this->_checkClientIdAndClientSecret($authorizationHeaderValue, $type, $serviceId);

        if ($credentialsValid) {
            switch ($grant_type) {
                case 'refresh_token':
                    $refresh_token = $json['refresh_token'];
                    return $this->_refreshToken($refresh_token, $type, $serviceId);
                case 'authorization_code':
                    $auth_code = $json['code'];
                    return $this->_redeemCodeForToken($auth_code, $type, $serviceId);
                default:
                    return $this->_httpFactory->buildResponse([
                        'error' => 'invalid_grant'
                    ], 400);
            }
        } else {
            return $this->_httpFactory->buildResponse([
                'error' => 'The Client ID or Client Secret you have provided is not valid.'
            ], 401);
        }
    }

    private function _refreshToken($refreshToken, $type, $serviceId) {
        $expires = $type === 'google' ? 'expires_in' : 'expires';

        $user = $this->_serviceUserDao->getUserByRefreshToken($refreshToken, $type, $serviceId);

        $auth_token = bin2hex(random_bytes(64));

        $token_data = [
            $serviceId => [
                $type => [
                    'access_token' => $auth_token,
                    'refresh_token' => $refreshToken,
                    'token_type' => 'bearer',
                    $expires => 3600
                ]
            ]
        ];

        $this->_serviceUserDao->updateUser($user['userId'], [
            'accessToken' => $token_data
        ]);

        return $this->_httpFactory->buildResponse($token_data[$type]);
    }

    private function _redeemCodeForToken($code, $type, $serviceId) {
        $expires = $type === 'google' ? 'expires_in' : 'expires';

        try {
            $user = $this->_serviceUserDao->getUserByAuthCode($code, $type, $serviceId);

            // todo: mix in some user data so that this isn't completely random?
            $auth_token = bin2hex(random_bytes(64));
            $refresh_token = bin2hex(random_bytes(16));

            $token_data = [
                $serviceId => [
                    $type => [
                        'access_token' => $auth_token,
                        'refresh_token' => $refresh_token,
                        'token_type' => 'bearer',
                        $expires => 3600
                    ]
                ]
            ];

            $this->_serviceUserDao->updateUser($user['userId'], [
                'authCode' => [
                    $serviceId => [
                        $type => ['redeemed' => true]
                    ]
                ],
                'accessToken' => $token_data
            ]);

            return $this->_httpFactory->buildResponse($token_data[$serviceId][$type]);
        } catch (DataItemNotFoundException $e) {
            return $this->_httpFactory->buildResponse(['message' => 'Auth code not found.'], 401);
        }
    }

    private function _generateAuthCodeForUser($user)
    {
        $data = $user['userId'].$user['firstName'].$user['lastName'].bin2hex(random_bytes(16));

        return hash("haval128,3", $data);
    }

    /**
     * Checks if the credentials are valid with the configured account linking on Alexa and out Convo service.
     *
     * @param $authorizationHeader
     * @param $type
     * @param $serviceId
     * @return bool
     */
    private function _checkClientIdAndClientSecret($authorizationHeader, $type, $serviceId) {
        $isValid = true;

        if ($type === 'amazon') {
            if (empty($authorizationHeader)) {
                $this->_logger->warning('No authorization header present.');
                $isValid = false;
            } else {
                $authType =  explode(" ", $authorizationHeader)[0] ?? '';
                $authType = strtolower($authType);

                if ($authType === 'basic') {
                    $encodedCredentials = explode(" ", $authorizationHeader)[1] ?? '';
                    $decodedClientIdAndClientSecret = base64_decode($encodedCredentials);
                    try {
                        $platform_config = $this->_convoServiceDataProvider->getServicePlatformConfig(
                            new RestSystemUser(), $serviceId, IPlatformPublisher::MAPPING_TYPE_DEVELOP
                        );

                        $amazonAccountLinkingClientId = $platform_config[$type]['account_linking_config']['client_id'] ?? 'unknown_configured_client_id';
                        $amazonAccountLinkingClientSecret = $platform_config[$type]['account_linking_config']['client_secret'] ?? 'unknown_configured_client_secret';
                        $configuredCredentials = $amazonAccountLinkingClientId . ":" . $amazonAccountLinkingClientSecret;

                        if ($decodedClientIdAndClientSecret !== $configuredCredentials) {
                            $this->_logger->warning('The Client ID or Client Secret you have provided is not valid.');
                            $isValid = false;
                        }
                    } catch (DataItemNotFoundException $e) {
                        $this->_logger->warning($e->getMessage());
                        $isValid = false;
                    }
                } else {
                    $this->_logger->warning('Invalid authorization header.');
                    $isValid = false;
                }
            }
        } else if ($type === 'google') {
            if (empty($authorizationHeader)) {
                $this->_logger->warning('No authorization header present.');
                $isValid = false;
            } else {
                $authType =  explode(" ", $authorizationHeader)[0] ?? '';
                $authType = strtolower($authType);

                if ($authType === 'basic') {
                    $encodedCredentials = explode(" ", $authorizationHeader)[1] ?? '';
                    $decodedClientIdAndClientSecret = base64_decode($encodedCredentials);
                    $this->_logger->info($decodedClientIdAndClientSecret);

                    // todo compare client_id and client_secret with Google
                    $isValid = true;
                }
            }
        }

        return $isValid;
    }

    public function __toString()
    {
        return get_class($this).'[]';
    }
}
